/*
Module : DENTRY.H
Purpose: Defines the interface to a number of classes to allow data entry of 
         the date/time classes
Created: PJN / DATE/1 / 20-02-1996
History: PJN / DATE/2 / 23-09-1996  Now uses new tabbed dialog method of 
                                    data entry

Copyright (c) 1996 by PJ Naughter.  
All rights reserved.

*/

#ifndef __DENTRY_H__


////////////////////////////////// Macros /////////////////////////////////////

#define __DENTRY_H__


////////////////////////////////// Consts /////////////////////////////////////

//flags used to control how the data entry dialogs are displayed

const DWORD DT_ONLYVALID         = 0x0001;   //only allows valid data to be entered
const DWORD DT_NOSECONDS         = 0x0002;   //do not show the second field
const DWORD DT_NODISPLAYCOMBO    = 0x0004;   //Only allows the date to be entered as a Day of Month
const DWORD DT_NOET_TIMEFRAME    = 0x0008;   //ET is not allowed as a TimeFrame
const DWORD DT_NOUCT_TIMEFRAME   = 0x0010;   //UCT is not allowed as a TimeFrame
const DWORD DT_NOLOCAL_TIMEFRAME = 0x0020;   //Local is not allowed as a TimeFrame




////////////// Id/Message Map consts //////////////////////////////////////////
#define DTIME_EDIT_CONTROL_ID 1000                                                   
#define UWM_BN_CLICKED       WM_USER + 1000
#define UWM_TIMEOFDAY_NOTIFY WM_USER + 1001
#define UWM_TIMEFRAME_NOTIFY WM_USER + 1002


////////////////////////////////// Includes ///////////////////////////////////
#include "datetime.h"


////////////////////////////// classes ////////////////////////////////////////
class CDateButton : public CButton
{
public:
  CDateButton();
  virtual ~CDateButton();

  void SetBuddy(CWnd* pBuddy);


protected:
  //{{AFX_VIRTUAL(CDateButton)
  public:
  virtual BOOL PreTranslateMessage(MSG* pMsg);
  //}}AFX_VIRTUAL

  //{{AFX_MSG(CDateButton)
  afx_msg void OnClicked();
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

  CWnd* m_pBuddy;
  BOOL m_bFirstCall;
  CToolTipCtrl m_ToolTip;
};


class AFX_EXT_CLASS CDateTimeControl : public CStatic
{
public:
  CDateTimeControl();
  virtual ~CDateTimeControl();

  BOOL SubclassEdit(UINT iCtlID, CWnd* pParentWnd);
  BOOL SubclassEdit(HWND hEdit);
  void SetFlags(DWORD dwFlags) { m_dwFlags = dwFlags; };

protected:
  //{{AFX_VIRTUAL(CDateTimeControl)
  //}}AFX_VIRTUAL

  //{{AFX_MSG(CDateTimeControl)
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

  BOOL AddEditButton();

  BOOL m_bSubclassed;
  CDateButton m_Edit;
  DWORD m_dwFlags;
};


class AFX_EXT_CLASS CDateControl : public CDateTimeControl
{
public:
//Constructors / Destructors
  CDateControl();
  virtual ~CDateControl();

//Accessors / Mutators
  void SetValue(const CDate& value);
  void GetValue(CDate& value) { value = m_Value; };
  
protected:
  //{{AFX_VIRTUAL(CDateControl)
  //}}AFX_VIRTUAL

  //{{AFX_MSG(CDateControl)
  afx_msg LRESULT OnEditClick(WPARAM, LPARAM);
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

  CDate m_Value;
};


class AFX_EXT_CLASS CLTimeSpanControl : public CDateTimeControl
{
public:
//Constructors / Destructors
  CLTimeSpanControl();
  virtual ~CLTimeSpanControl();

//Accessors / Mutators
  void SetValue(const CLTimeSpan& value);
  void GetValue(CLTimeSpan& value) { value = m_Value; };

protected:
  //{{AFX_VIRTUAL(CLTimeSpanControl)
  //}}AFX_VIRTUAL

  //{{AFX_MSG(CLTimeSpanControl)
  afx_msg LRESULT OnEditClick(WPARAM, LPARAM);
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

  CLTimeSpan m_Value;
};


class AFX_EXT_CLASS CLTimeOfDayControl : public CDateTimeControl
{
public:
//Constructors / Destructors
  CLTimeOfDayControl();
  virtual ~CLTimeOfDayControl();

//Accessors / Mutators
  void SetValue(const CLTimeOfDay& value);
  void GetValue(CLTimeOfDay& value) { value = m_Value; };

protected:
  //{{AFX_VIRTUAL(CCLTimeOfDayControl)
  //}}AFX_VIRTUAL

  //{{AFX_MSG(CLTimeOfDayControl)
  afx_msg LRESULT OnEditClick(WPARAM, LPARAM);
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

  CLTimeOfDay m_Value;
};


class AFX_EXT_CLASS CLDateControl : public CDateTimeControl
{
public:
//Constructors / Destructors
  CLDateControl();
  virtual ~CLDateControl();

//Accessors / Mutators
  void SetValue(const CLDate& value);
  void GetValue(CLDate& value) { value = m_Value; };

protected:
  //{{AFX_VIRTUAL(CLDateControl)
  //}}AFX_VIRTUAL

  //{{AFX_MSG(CLDateControl)
  afx_msg LRESULT OnEditClick(WPARAM, LPARAM);
  //}}AFX_MSG

  DECLARE_MESSAGE_MAP()

  CLDate m_Value;
};


class CDatePropertyPage : public CPropertyPage
{
public:
	//Constructors / Destructors
	CDatePropertyPage();
  CDatePropertyPage(UINT nIDTemplate, UINT nIDCaption = 0);
	~CDatePropertyPage();

	//functions
	void SetFlags(DWORD dwFlags) { m_dwFlags = dwFlags; };

	//Accessors / Mutators
	void SetDate(const CDate& date) { m_Date = date; };
	CDate GetDate() const {return m_Date; };


protected:
	//{{AFX_DATA(CDatePropertyPage)
	//}}AFX_DATA
  
	//{{AFX_VIRTUAL(CDatePropertyPage)
	//}}AFX_VIRTUAL

	DECLARE_DYNCREATE(CDatePropertyPage)

	//{{AFX_MSG(CDatePropertyPage)
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
  
  DWORD m_dwFlags;
  CDate m_Date;
};




class CDatePropertySheet : public CPropertySheet
{
public:
	//Constructors / Destructors
  CDatePropertySheet();
	CDatePropertySheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	CDatePropertySheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	virtual ~CDatePropertySheet();

protected:
  //{{AFX_DATA(CDatePropertySheet)
	//}}AFX_DATA
  
	//{{AFX_VIRTUAL(CDatePropertySheet)
	public:
	//}}AFX_VIRTUAL

	DECLARE_DYNCREATE(CDatePropertySheet)

	//{{AFX_MSG(CDatePropertySheet)
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
  
  DWORD m_dwFlags;
};



class CCDateDOMPage : public CDatePropertyPage
{
public:
	//Constructors / Destructors
	CCDateDOMPage();
	~CCDateDOMPage();

protected:
	//{{AFX_DATA(CCDateDOMPage)
	enum { IDD = IDD_CDATE_DOM };
	CSpinButtonCtrl	m_SpinYear;
	CSpinButtonCtrl	m_SpinDay;
	UINT	m_nDay;
	long	m_lYear;
	BOOL	m_bInvalid;
	//}}AFX_DATA
  WORD m_nMonth;

	//{{AFX_VIRTUAL(CCDateDOMPage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	DECLARE_DYNCREATE(CCDateDOMPage)

	//{{AFX_MSG(CCDateDOMPage)
	afx_msg void OnInvalid();
	virtual BOOL OnInitDialog();
	afx_msg void OnToday();
	afx_msg void OnSelchangeDisplay();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};



class CTimeFramePage : public CDatePropertyPage
{
public:
	//Constructors / Destructors
	CTimeFramePage();
	~CTimeFramePage();

	//Accessors / Mutators
	void SetTimeFrame(TimeFrame tf) { m_TimeFrame = tf; };
	TimeFrame GetTimeFrame() const { return m_TimeFrame; };

  void SetInvalid(BOOL bInvalid) { m_bInvalid = bInvalid; };


protected:
	//{{AFX_DATA(CTimeFramePage)
	enum { IDD = IDD_TIMEFRAME };
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CTimeFramePage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnSetActive();
	//}}AFX_VIRTUAL

	DECLARE_DYNCREATE(CTimeFramePage)

	//{{AFX_MSG(CTimeFramePage)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG

	void DisableIfInvalid();

	DECLARE_MESSAGE_MAP()

	TimeFrame m_TimeFrame;
  BOOL m_bInvalid;
};



class CCDateDOYPage : public CDatePropertyPage
{
public:
	// Constructors / Destructors
	CCDateDOYPage();
	~CCDateDOYPage();


protected:
	//{{AFX_DATA(CCDateDOYPage)
	enum { IDD = IDD_CDATE_DOY };
	CSpinButtonCtrl	m_SpinYear;
	CSpinButtonCtrl	m_SpinDay;
	UINT	m_nDay;
	BOOL	m_bInvalid;
	long	m_lYear;
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CCDateDOYPage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	DECLARE_DYNCREATE(CCDateDOYPage)

	//{{AFX_MSG(CCDateDOYPage)
	afx_msg void OnInvalid();
	virtual BOOL OnInitDialog();
	afx_msg void OnToday();
	afx_msg void OnSelchangeDisplay();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};



class CCDateWOMPage : public CDatePropertyPage
{
public:
	//Constructors / Destructors
	CCDateWOMPage();
	~CCDateWOMPage();

protected:
	DECLARE_DYNCREATE(CCDateWOMPage)

	//{{AFX_DATA(CCDateWOMPage)
	enum { IDD = IDD_CDATE_WOM };
	CSpinButtonCtrl	m_SpinYear;
	BOOL	m_bInvalid;
	long	m_lYear;
	//}}AFX_DATA
  WORD m_nDayOfWeek;
  WORD m_nWeek;
  WORD m_nMonth;

	//{{AFX_VIRTUAL(CCDateWOMPage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCDateWOMPage)
	afx_msg void OnInvalid();
	virtual BOOL OnInitDialog();
	afx_msg void OnToday();
	afx_msg void OnSelchangeDisplay();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};



class CCDateWOYPage : public CDatePropertyPage
{
public:
	//Constructors / Destructors
	CCDateWOYPage();
	~CCDateWOYPage();

protected:
	DECLARE_DYNCREATE(CCDateWOYPage)

	//{{AFX_DATA(CCDateWOYPage)
	enum { IDD = IDD_CDATE_WOY };
	CSpinButtonCtrl	m_SpinYear;
	CSpinButtonCtrl	m_SpinWeek;
	BOOL	m_bInvalid;
	long	m_lYear;
  int m_nWeek;
	//}}AFX_DATA
  WORD m_nDayOfWeek;
  

	//{{AFX_VIRTUAL(CCDateWOYPage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCDateWOYPage)
  afx_msg void OnInvalid();
	virtual BOOL OnInitDialog();
	afx_msg void OnToday();
	afx_msg void OnSelchangeDisplay();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};



class CCLTimeOfDayHMSMPage : public CDatePropertyPage
{
public:
	//Constructors / Destructors
	CCLTimeOfDayHMSMPage();
	~CCLTimeOfDayHMSMPage();

  void SetShowInvalidCheck(BOOL bShowInvalid) { m_bShowInvalid = bShowInvalid; }; 
  void MarkAsValid();

	//Accessors / Mutators
	void SetTimeOfDay(const CLTimeOfDay& Tod) { m_Tod = Tod; };
	CLTimeOfDay GetTimeOfDay() const { return m_Tod; };


protected:
	DECLARE_DYNCREATE(CCLTimeOfDayHMSMPage)

	//{{AFX_DATA(CCLTimeOfDayHMSMPage)
	enum { IDD = IDD_CLTIMEOFDAY_HMSM };
	CSpinButtonCtrl	m_SpinSecond;
	CSpinButtonCtrl	m_SpinMinute;
	CSpinButtonCtrl	m_SpinHour;
	UINT	m_nHour;
	BOOL	m_bInvalid;
	UINT	m_nMinute;
	UINT	m_nSecond;
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CCLTimeOfDayHMSMPage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnSetActive();
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCLTimeOfDayHMSMPage)
	afx_msg void OnInvalid();
	virtual BOOL OnInitDialog();
	afx_msg void OnNow();
	afx_msg void OnMidday();
	afx_msg void OnMidnight();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

  BOOL m_bShowInvalid;
  CLTimeOfDay m_Tod;
};



class CCLTimeSpanDHMSMPage : public CDatePropertyPage
{
public:
	//Constructors / Destructors
	CCLTimeSpanDHMSMPage();
	~CCLTimeSpanDHMSMPage();

	//Accessors / Mutators
	void SetTimeSpan(const CLTimeSpan& Ts) { m_Ts = Ts; };
	CLTimeSpan GetTimeSpan() const { return m_Ts; };


protected:
	DECLARE_DYNCREATE(CCLTimeSpanDHMSMPage)

	//{{AFX_DATA(CCLTimeSpanDHMSMPage)
	enum { IDD = IDD_CLTIMESPAN_DHMSM };
	CSpinButtonCtrl	m_SpinSecond;
	CSpinButtonCtrl	m_SpinMinute;
	CSpinButtonCtrl	m_SpinHour;
	CSpinButtonCtrl	m_SpinDay;
	long	m_lDay;
	UINT	m_nHour;
	BOOL	m_bInvalid;
	UINT	m_nMinute;
	UINT	m_nSecond;
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CCLTimeSpanDHMSMPage)
	public:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCLTimeSpanDHMSMPage)
	afx_msg void OnInvalid();
	virtual BOOL OnInitDialog();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

  CLTimeSpan m_Ts;
};



enum CDateCurrentPage
{
	DOMPage,
	DOYPage,
	WOMPage,
	WOYPage
};



class AFX_EXT_CLASS CCDateSheet : public CDatePropertySheet
{
public:
	//Constructors / Destructors
	CCDateSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	CCDateSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	virtual ~CCDateSheet();

  //Accessors / Mutators
  CDate GetCDate() const;
  void SetCDate(const CDate& date);


protected:
	DECLARE_DYNAMIC(CCDateSheet)

	//{{AFX_VIRTUAL(CCDateSheet)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCDateSheet)
	afx_msg void OnDOMFormat();
	afx_msg void OnDOYFormat();
	afx_msg void OnWOMFormat();
	afx_msg void OnWOYFormat();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

  CCDateDOMPage m_DOMPage;
  CCDateWOMPage m_WOMPage;
  CCDateWOYPage m_WOYPage;
  CCDateDOYPage m_DOYPage;
  CDateCurrentPage m_DateCurrPage;
  CDate m_Date;
};



class AFX_EXT_CLASS CCLDateSheet : public CDatePropertySheet
{
public:
	//Constructors / Destructors
	CCLDateSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	CCLDateSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	virtual ~CCLDateSheet();

  //Accessors / Mutators
  CLDate GetCLDate() const;
  void SetCLDate(const CLDate& date);


protected:
	DECLARE_DYNAMIC(CCLDateSheet)

	//{{AFX_VIRTUAL(CCLDateSheet)
	//}}AFX_VIRTUAL

  afx_msg LRESULT OnTimeOfDayNotify(WPARAM wParam, LPARAM lParam);
  afx_msg LRESULT OnTimeFrameNotify(WPARAM wParam, LPARAM lParam);
	//{{AFX_MSG(CCLDateSheet)
	afx_msg void OnDOMFormat();
	afx_msg void OnDOYFormat();
	afx_msg void OnWOMFormat();
	afx_msg void OnWOYFormat();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

  CCDateDOMPage        m_DOMPage;
	CCDateWOMPage        m_WOMPage;
	CCDateWOYPage        m_WOYPage;
	CCDateDOYPage        m_DOYPage;
	CCLTimeOfDayHMSMPage m_HMSMPage;
  CTimeFramePage       m_TimeFramePage;
	CDateCurrentPage     m_DateCurrPage;
  CLDate m_Ldate;
};



class AFX_EXT_CLASS CCLTimeOfDaySheet : public CDatePropertySheet
{
public:
	//Constructors /Destructors
	CCLTimeOfDaySheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	CCLTimeOfDaySheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	virtual ~CCLTimeOfDaySheet();

  //Accessors / Mutators
  CLTimeOfDay GetCLTimeOfDay() const { return m_HMSMPage.GetTimeOfDay(); };
  void SetCLTimeOfDay(const CLTimeOfDay& Tod);

protected:
	DECLARE_DYNAMIC(CCLTimeOfDaySheet)

	//{{AFX_VIRTUAL(CCLTimeOfDaySheet)
	//}}AFX_VIRTUAL

  afx_msg LRESULT OnTimeOfDayNotify(WPARAM wParam, LPARAM lParam);
	//{{AFX_MSG(CCLTimeOfDaySheet)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

  CCLTimeOfDayHMSMPage m_HMSMPage;
};



class AFX_EXT_CLASS CCLTimeSpanSheet : public CDatePropertySheet
{
public:
	//Constructors / Destructors
	CCLTimeSpanSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	CCLTimeSpanSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage, DWORD dwFlags);
	virtual ~CCLTimeSpanSheet();

  //Accessors / Mutators
  CLTimeSpan GetCLTimeSpan() const { return m_DHMSMPage.GetTimeSpan(); };
  void SetCLTimeSpan(const CLTimeSpan& Ts);

protected:
	DECLARE_DYNAMIC(CCLTimeSpanSheet)

	//{{AFX_VIRTUAL(CCLTimeSpanSheet)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCLTimeSpanSheet)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

  CCLTimeSpanDHMSMPage m_DHMSMPage;
};





class AFX_EXT_CLASS CDTimeReminderDlg : public CDialog
{
public:
  //Constructors / Destructors
	CDTimeReminderDlg(CWnd* pParent = NULL);   // standard constructor


protected:
	//{{AFX_VIRTUAL(CDTimeReminderDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL


	//{{AFX_DATA(CDTimeReminderDlg)
	enum { IDD = IDD_DTIME_ABOUT };
	//}}AFX_DATA


	//{{AFX_MSG(CDTimeReminderDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};




//MFC Data exchange routines

void AFX_EXT_API DDX_CDateControl(CDataExchange* pDX, int nIDC, CDateControl& rCDateControl, DWORD dwFlags);
void AFX_EXT_API DDX_CLTimeSpanControl(CDataExchange* pDX, int nIDC, CLTimeSpanControl& rCLTimeSpanControl, DWORD dwFlags);
void AFX_EXT_API DDX_CLTimeOfDayControl(CDataExchange* pDX, int nIDC, CLTimeOfDayControl& rCLTimeOfDayControl, DWORD dwFlags);
void AFX_EXT_API DDX_CLDateControl(CDataExchange* pDX, int nIDC, CLDateControl& rCLDateControl, DWORD dwFlags);

void AFX_EXT_API DDX_CDate(CDataExchange* pDX, CDateControl& rCDateControl, CDate& value);
void AFX_EXT_API DDV_MinMaxCDate(CDataExchange* pDX, CDate& value, CDate& minVal, CDate& maxVal);
void AFX_EXT_API DDV_GreaterThanCDate(CDataExchange* pDX, CDate& value, CDate& minVal);
void AFX_EXT_API DDV_LessThanCDate(CDataExchange* pDX, CDate& value, CDate& maxVal);

void AFX_EXT_API DDX_CLTimeSpan(CDataExchange* pDX, CLTimeSpanControl& rCLTimeSpanControl, CLTimeSpan& value);
void AFX_EXT_API DDV_MinMaxCLTimeSpan(CDataExchange* pDX, CLTimeSpan& value, CLTimeSpan& minVal, CLTimeSpan& maxVal);
void AFX_EXT_API DDV_GreaterThanCLTimeSpan(CDataExchange* pDX, CLTimeSpan& value, CLTimeSpan& minVal);
void AFX_EXT_API DDV_LessThanCLTimeSpan(CDataExchange* pDX, CLTimeSpan& value, CLTimeSpan& maxVal);

void AFX_EXT_API DDX_CLTimeOfDay(CDataExchange* pDX, CLTimeOfDayControl& rCLTimeOfDayControl, CLTimeOfDay& value);
void AFX_EXT_API DDV_MinMaxCLTimeOfDay(CDataExchange* pDX, CLTimeOfDay& value, CLTimeOfDay& minVal, CLTimeOfDay& maxVal);
void AFX_EXT_API DDV_GreaterThanCLTimeOfDay(CDataExchange* pDX, CLTimeOfDay& value, CLTimeOfDay& minVal);
void AFX_EXT_API DDV_LessThanCLTimeOfDay(CDataExchange* pDX, CLTimeOfDay& value, CLTimeOfDay& maxVal);

void AFX_EXT_API DDX_CLDate(CDataExchange* pDX, CLDateControl& rCLDateControl, CLDate& value);
void AFX_EXT_API DDV_MinMaxCLDate(CDataExchange* pDX, CLDate& value, CLDate& minVal, CLDate& maxVal);
void AFX_EXT_API DDV_GreaterThanCLDate(CDataExchange* pDX, CLDate& value, CLDate& minVal);
void AFX_EXT_API DDV_LessThanCLDate(CDataExchange* pDX, CLDate& value, CLDate& maxVal);

void AFX_EXT_API DDX_CBData(CDataExchange* pDX, int nIDC, DWORD& dwData);


#endif //__DENTRY_H__




                                        
